@@ -10,7 +10,7 @@
    sure your hard disk is already infested with copies of this file,
    but if not, mail me and I'll send you one.
 
-   $Id: procinfo.c,v 1.16 1994/05/21 06:46:52 svm Exp svm $
+   $Id: procinfo.c,v 1.19 1994/08/11 22:45:42 svm Exp svm $
  */
 
 #include <stdio.h>
@@ -25,7 +25,7 @@
 #include <termios.h>
 #include <sys/ioctl.h>
 
-#define VERSION		"0.4 (1994-05-21)"
+#define VERSION		"0.5 (1994-08-12)"
 #define PROC_DIR	"/proc/"	/* Needs the trailing slash. */
 
 #define ISSTR(s)	(!strcmp(s, type))
@@ -43,19 +43,17 @@
 #endif
 #define MAX_DEV		MAX(MAX_CHRDEV, MAX_BLKDEV)
 
-typedef signed long ul;
-
 struct info
 {
-    ul  uptime;
-    ul  m_to, m_us, m_fr, m_sh, m_bu;
-    ul  s_to, s_us, s_fr;
-    ul  cpu_user, cpu_nice, cpu_sys, cpu_idle;
-    ul  disk[4];
-    ul  pgin, pgout, swin, swout;
-    ul  intr[16];
-    ul  old_intr;
-    ul  ctxt;
+    long uptime;
+    long m_to, m_us, m_fr, m_sh, m_bu;
+    long s_to, s_us, s_fr;
+    long cpu_user, cpu_nice, cpu_sys, cpu_idle;
+    long disk[4];
+    long pgin, pgout, swin, swout;
+    long intr[16];
+    long old_intr;
+    long ctxt;
 };
 
 struct info new, old;
@@ -79,6 +77,7 @@ int fs = 0, redrawn = 0;
 int show_moddev = 0;
 int show_all = 0;
 int show_diff = 0;
+int show_new_mem = 0;
 int irq_array = 0;
 
 static char booted[40];
@@ -97,24 +96,29 @@ myfopen (char *name)
 }
 
 /* Note: we're using a static char array, so use this only once per printf. */
-char *
+inline char *
 hms (long t)
 {
-    int h, m, s;
-    static char buf[16];
+    int d, h, m, s;
+    static char buf[22];
 
+    d = (int) (t / 8640000);
+    t = t - (long) (d * 8640000);
     h = (int) (t / 360000);
     t = t - (long) (h * 360000);
     m = (int) (t / 6000);
     t = t - (long) (m * 6000);
     s = (int) (t / 100);
     t = t - (long) (s * 100);
-    sprintf (buf, "%3d:%02d:%02d.%02d", h, m, s, (int) t);
+    if (d > 0)
+	sprintf (buf, "%3dd %2d:%02d:%02d.%02d", d, h, m, s, (int) t);
+    else
+	sprintf (buf, "     %2d:%02d:%02d.%02d", h, m, s, (int) t);
     return buf;
 }
 
 /* Note: we're using a static char array, so use this only once per printf. */
-char *
+inline char *
 perc (long i, long t)
 {
     int v;
@@ -183,14 +187,14 @@ int
 main (int ac, char **av)
 {
     FILE *loadavgfp, *meminfofp, *modulesfp, *statfp, *uptimefp, *versionfp,
-       *devicesfp, *filesystemsfp;
-    char line[1024];
-    char version[1024];
+       *devicesfp, *filesystemsfp, *interruptsfp;
+    char line[1024], version[1024], tmpbuf[1024];
     int sl = 5;
     int getoptopt;
     char outbuf[4096];
+    char *p;			/* random pointer */
 
-    while ((getoptopt = getopt (ac, av, "fn:madhv")) != EOF)
+    while ((getoptopt = getopt (ac, av, "fn:madDhv")) != EOF)
     {
 	switch (getoptopt)
 	{
@@ -206,8 +210,12 @@ main (int ac, char **av)
 	case 'a':
 	    show_all = 1;
 	    break;
+	case 'D':
+	    show_new_mem = fs = 1;	/* show new mamory & page stats, diff 
+
+					   on rest */
 	case 'd':
-	    show_diff = 1;
+	    show_diff = fs = 1;
 	    break;
 	case 'v':
 	    printf ("This is procinfo version " VERSION "\n");
@@ -215,17 +223,18 @@ main (int ac, char **av)
 	case 'h':
 	default:
 	    printf ("procinfo version %s\n"
-		    "usage: %s [-fmavdh] [-nN]\n"
+		    "usage: %s [-fmadDvh] [-nN]\n"
 		    "\n"
 		    "\t-f\trun full screen\n"
 		    "\t-nN\tpause N second between updates (implies -f)\n"
 		    "\t-m\tdisplay module and device info\n"
 		    "\t-a\tdisplay all info\n"
-		    "\t-d\tshow differences rather than totals\n"
+		  "\t-d\tshow differences rather than totals (implies -f)\n"
+	       "\t-D\tshow current memory/swap usage, differences on rest\n"
 		    "\t-v\tprint version info\n"
 		    "\t-h\tprint this help\n",
 		    VERSION, av[0]);
-	    exit (1);
+	    exit (getoptopt == 'h' ? 0 : 1);
 	}
     }
     if (sl == 0)
@@ -251,9 +260,16 @@ main (int ac, char **av)
     signal (SIGTSTP, tstp);
     signal (SIGCONT, cont);
 
+    gethostname (tmpbuf, 65);
     versionfp = myfopen (PROC_DIR "version");
-    fgets (version, sizeof (version), versionfp);
-    version[strlen (version) - 1] = '\0';
+    fgets (line, sizeof (version), versionfp);
+   /* Junk everything after '#xx' or it'll get too long with later kernels. */
+    p = strchr (line, '#');
+    p = strchr (p, ' ');
+    strncpy (version, line, p - line);
+    sprintf (&version[p - line], "%*s [%s]",
+	     fs ? -(co - strlen (tmpbuf) - strlen (version) - 3) : 0, "",
+	     tmpbuf);
     fclose (versionfp);
 
     uptimefp = myfopen (PROC_DIR "uptime");
@@ -261,9 +277,10 @@ main (int ac, char **av)
     meminfofp = myfopen (PROC_DIR "meminfo");
     statfp = myfopen (PROC_DIR "stat");
     modulesfp = myfopen (PROC_DIR "modules");
-   /* These two may be missing, so check for NULL later. */
+   /* These three may be missing, so check for NULL later. */
     devicesfp = fopen (PROC_DIR "devices", "r");
     filesystemsfp = fopen (PROC_DIR "filesystems", "r");
+    interruptsfp = fopen (PROC_DIR "interrupts", "r");
 
    /* See what the intr line in /proc/stat says. */
     while (fgets (line, sizeof (line), statfp))
@@ -281,7 +298,7 @@ main (int ac, char **av)
 	{
 	    time_t btime;
 
-	    btime = (time_t) atol(&line[6]);
+	    btime = (time_t) atol (&line[6]);
 	    strftime (booted, sizeof (booted), "%c", localtime (&btime));
 	    continue;
 	}
@@ -294,7 +311,7 @@ main (int ac, char **av)
     {
 	int i;
 	char loadavg[32];
-	
+
 	if (redrawn)
 	{
 	    redrawn = 0;
@@ -307,7 +324,7 @@ main (int ac, char **av)
 
 	fseek (uptimefp, 0L, SEEK_SET);
 	fgets (line, sizeof (line), uptimefp);
-	new.uptime = (ul) (atof (strtok (line, " ")) * 100.0);
+	new.uptime = (long) (atof (strtok (line, " ")) * 100.0);
 	fgets (line, sizeof (line), uptimefp);	/* For libc 4.4; more below. */
 
 	fseek (loadavgfp, 0L, SEEK_SET);
@@ -329,16 +346,31 @@ main (int ac, char **av)
 	    new.m_fr = VAL / 1024;
 	    new.m_sh = VAL / 1024;
 	    new.m_bu = VAL / 1024;
-	    printf ("Mem:    %12ld%12ld%12ld%12ld%12ld\n",
-		    DIFF (m_to), DIFF (m_us), DIFF (m_fr), DIFF (m_sh),
-		    DIFF (m_bu));
+	    if (show_new_mem)
+	    {
+		printf ("Mem:    %12ld%12ld%12ld%12ld%12ld\n",
+			new.m_to, new.m_us, new.m_fr, new.m_sh, new.m_bu);
+	    }
+	    else
+	    {
+		printf ("Mem:    %12ld%12ld%12ld%12ld%12ld\n",
+			DIFF (m_to), DIFF (m_us), DIFF (m_fr), DIFF (m_sh), DIFF (m_bu));
+	    }
 	    fgets (line, sizeof (line), meminfofp);
 	    strtok (line, " ");
 	    new.s_to = VAL / 1024;
 	    new.s_us = VAL / 1024;
 	    new.s_fr = VAL / 1024;
-	    printf ("Swap:   %12ld%12ld%12ld\n\n",
-		    DIFF (s_to), DIFF (s_us), DIFF (s_fr));
+	    if (show_new_mem)
+	    {
+		printf ("Swap:   %12ld%12ld%12ld\n\n",
+			new.s_to, new.s_us, new.s_fr);
+	    }
+	    else
+	    {
+		printf ("Swap:   %12ld%12ld%12ld\n\n",
+			DIFF (s_to), DIFF (s_us), DIFF (s_fr));
+	    }
 	    fgets (line, sizeof (line), meminfofp);
 	}
 
@@ -386,13 +418,9 @@ main (int ac, char **av)
 	    }
 	    else if (ISSTR ("ctxt"))
 		new.ctxt = VAL;
-#if 0
-	    else if (ISSTR ("btime"))
-		new.btime = (time_t) VAL;
-#endif
 	}
 
-	printf ("Bootup: %s\tLoad average: %s\n\n", booted, loadavg);
+	printf ("Bootup: %s    Load average: %s\n\n", booted, loadavg);
 
 	if (!show_moddev || show_all)
 	{
@@ -414,48 +442,67 @@ main (int ac, char **av)
 
 	    printf ("user  : %s %s\t",
 		    hms (DIFF (cpu_user)), perc (DIFF (cpu_user), elapsed));
-	    printf ("page in : %8ld\t", DIFF (pgin));
+	    printf ("    page in : %8ld\t", DIFF (pgin));
 	    if (new.disk[0])
-		printf ("disk 1: %8ld\n", DIFF (disk[0]));
+		printf ("  disk 1: %8ld\n", DIFF (disk[0]));
 	    else
 		putchar ('\n');
 
 	    printf ("nice  : %s %s\t",
 		    hms (DIFF (cpu_nice)), perc (DIFF (cpu_nice), elapsed));
-	    printf ("page out: %8ld\t", DIFF (pgout));
+	    printf ("    page out: %8ld\t", DIFF (pgout));
 	    if (new.disk[1])
-		printf ("disk 2: %8ld\n", DIFF (disk[1]));
+		printf ("  disk 2: %8ld\n", DIFF (disk[1]));
 	    else
 		putchar ('\n');
 
 	    printf ("system: %s %s\t",
 		    hms (DIFF (cpu_sys)), perc (DIFF (cpu_sys), elapsed));
-	    printf ("swap in : %8ld\t", DIFF (swin));
+	    printf ("    swap in : %8ld\t", DIFF (swin));
 	    if (new.disk[2])
-		printf ("disk 3: %8ld\n", DIFF (disk[2]));
+		printf ("  disk 3: %8ld\n", DIFF (disk[2]));
 	    else
 		putchar ('\n');
 
 	    printf ("idle  : %s %s\t",
 		    hms (DIFF (cpu_idle)), perc (DIFF (cpu_idle), elapsed));
-	    printf ("swap out: %8ld\t", DIFF (swout));
+	    printf ("    swap out: %8ld\t", DIFF (swout));
 	    if (new.disk[3])
-		printf ("disk 4: %8ld\n", DIFF (disk[3]));
+		printf ("  disk 4: %8ld\n", DIFF (disk[3]));
 	    else
 		putchar ('\n');
 
-	    printf ("uptime: %s\t\tcontext : %8ld", hms (new.uptime),
+	    printf ("uptime: %s\t    context : %8ld", hms (new.uptime),
 		    DIFF (ctxt));
 
 	    if (irq_array)
 	    {
+		char irq_label[22][16];
+
+		if (interruptsfp)
+		{
+		    int i;
+
+		    for (i = 0; i < 16; i++)
+			irq_label[i][0] = '\0';
+
+		    fseek (interruptsfp, 0L, SEEK_SET);
+		    while (fgets (line, sizeof (line), interruptsfp))
+		    {
+			i = atol (strtok (line, ":"));
+			strncpy (irq_label[i], &line[15], 20);
+			irq_label[i][strlen (irq_label[i]) - 1] = '\0';
+			if (line[13] == '+')
+			    strcat (irq_label[i], "+");
+		    }
+		}
+
 		fputs ("\n\n", stdout);
-		for (i = 0; i < 4; i++)
-		    printf ("irq %2d: %9ld  irq %2d: %9ld  "
-			    "irq %2d: %9ld  irq %2d: %9ld\n",
-			    i, DIFF (intr[i]), i + 4, DIFF (intr[i + 4]),
-			    i + 8, DIFF (intr[i + 8]), i + 12,
-			    DIFF (intr[i + 12]));
+		for (i = 0; i < 8; i++)
+		    printf ("irq %2d: %9ld %-21s "
+			    "irq %2d: %9ld %-21s\n",
+			    i, DIFF (intr[i]), irq_label[i],
+			    i + 8, DIFF (intr[i + 8]), irq_label[i + 8]);
 	    }
 	    else
 		printf ("\tinterrupts: %8ld\n", DIFF (old_intr));
@@ -468,33 +515,40 @@ main (int ac, char **av)
 	    fseek (modulesfp, 0L, SEEK_SET);
 	    if (show_all)
 		putchar ('\n');
-	    printf ("Modules:      size\n");
+	    printf ("Modules:\n");
 	    while (fgets (line, sizeof (line), modulesfp))
 	    {
 		char *mod;
 		long pg;
-		char *status;
+		char *status, *tmp;
+		int used = 0;
 
+		tmp = strdup (line);
 		barf += 20;
 		mod = strtok (line, " ");
 		pg = atol (strtok (NULL, " ")) * 4;
 		status = strtok (NULL, ")");
-		printf ("%-12s%c %4ld  ",
-			mod, status ? status[2] : ' ', pg);
+		if (strchr (tmp, ']'))
+		    used++;
+		printf ("%c%4ld %1s%-10s   ",
+			status ? status[2] : ' ', pg,
+			used ? "*" : " ", mod);
+		free (tmp);
 	    }
-	    if (co && (barf % co))
-		printf ("%s\n", fs ? ce : "");
+/*          if (co && (barf % co)) */
+	    printf ("%s\n", fs ? ce : "");
 
 	    if (devicesfp)
 	    {
 		int maj[2][MAX_DEV];
 		char *n[2][MAX_DEV];
-		int count[2] = {0, 0};
+		int count[2] =
+		{0, 0};
 		int which = 0, i, j;
 
 		memset (n, 0, sizeof (n));
 		fseek (devicesfp, 0L, SEEK_SET);
-		printf ("%s\nCharacter Devices:                    "
+		printf ("%s\nCharacter Devices:                      "
 			"Block Devices:\n",
 			fs ? ce : "");
 		while (fgets (line, sizeof (line), devicesfp))
@@ -524,14 +578,14 @@ main (int ac, char **av)
 		{
 		    if (n[CDRV][i])
 		    {
-			printf ("%2d %-16s", maj[CDRV][i], n[CDRV][i]);
+			printf ("%2d %-16s ", maj[CDRV][i], n[CDRV][i]);
 			free (n[CDRV][i]);
 		    }
 		    else
 			fputs ("                    ", stdout);
 		    if (n[CDRV][i + j])
 		    {
-			printf ("%2d %-16s",
+			printf ("%2d %-16s ",
 				maj[CDRV][i + j], n[CDRV][i + j]);
 			free (n[CDRV][i + j]);
 		    }
@@ -539,14 +593,14 @@ main (int ac, char **av)
 			fputs ("                    ", stdout);
 		    if (n[BDRV][i])
 		    {
-			printf ("%2d %-16s", maj[BDRV][i], n[BDRV][i]);
+			printf ("%2d %-16s ", maj[BDRV][i], n[BDRV][i]);
 			free (n[BDRV][i]);
 		    }
 		    else
-			fputs ("                    ", stdout);
+			fputs ("                     ", stdout);
 		    if (n[BDRV][i + j])
 		    {
-			printf ("%2d %-16s",
+			printf ("%2d %-16s ",
 				maj[BDRV][i + j], n[BDRV][i + j]);
 			free (n[BDRV][i + j]);
 		    }
@@ -560,7 +614,8 @@ main (int ac, char **av)
 	    {
 		barf = 0;
 		fseek (filesystemsfp, 0L, SEEK_SET);
-		printf ("\nFile Systems:\n");
+		printf ("%s\n", fs ? ce : "");
+		printf ("File Systems:%s\n", fs ? ce : "");
 		while (fgets (line, sizeof (line), filesystemsfp))
 		{
 		    char *fs;
