@@ -8,7 +8,7 @@
 
    -svm
 
-  $Id: term.c,v 1.4 1994/05/08 00:19:02 svm Exp svm $
+  $Id: term.c,v 1.5 1994/05/19 01:24:15 svm Exp $
 */
 
 #include <stdio.h>
@@ -63,6 +63,8 @@ my_tgets (char *te)
     char *temp;
 
    /* Extract information we will use. */
-    temp = tgetstr (te, BUFFADDR);
-    return temp;
+    if ((temp = tgetstr (te, BUFFADDR)) != NULL)
+	return temp;
+    else
+	return "";
 }
